"""
    Data types used by Discord Interaction model

    Mostly enumerates and flags.
"""
from dataclasses import dataclass
from enum import Enum, IntEnum, IntFlag


#############################################################
#   Global
#############################################################


Snowflake = str  # I know this is an int, but API returns as a str


@dataclass
class DiscordDataClass():
    pass


#############################################################
#   Interaction
#############################################################

class ApplicationFlags(IntFlag):
    """
        Application flags
    """
    GATEWAY_PRESENCE = 1 << 12
    GATEWAY_PRESENCE_LIMITED = 1 << 13
    GATEWAY_GUILD_MEMBERS = 1 << 14
    GATEWAY_GUILD_MEMBERS_LIMITED = 1 << 15
    VERIFICATION_PENDING_GUILD_LIMIT = 1 << 16
    EMBEDDED = 1 << 17
    GATEWAY_MESSAGE_CONTENT = 1 << 18
    GATEWAY_MESSAGE_CONTENT_LIMITED = 1 << 19


class ApplicationCommandType(IntEnum):
    """
        Application Command Type

        Controls how the command will be invoked.
    """
    CHAT_INPUT = 1
    USER = 2
    MESSAGE = 3


class ApplicationCommandOptionType(IntEnum):
    """
        Application Command Option Type

        Specify the parameter data type for the command.
    """
    SUB_COMMAND = 1
    SUB_COMMAND_GROUP = 2
    STRING = 3
    INTEGER = 4
    BOOLEAN = 5
    USER = 6
    CHANNEL = 7
    ROLE = 8
    MENTIONABLE = 9
    NUMBER = 10
    ATTACHMEHT = 11


class InteractionType(IntEnum):
    """
        Interaction Type

        Interaction behavior identificator.
    """
    PING = 1
    APPLICATION_COMMAND = 2
    MESSAGE_COMPONENT = 3
    APPLICATION_COMMAND_AUTOCOMPLETE = 4
    MODAL_SUBMIT = 5


class InteractionCallbackType(IntEnum):
    """
        Interaction Callback Type

        Use when responding an interaction.
    """
    PONG = 1
    CHANNEL_MESSAGE_WITH_SOURCE = 4
    DEFERRED_CHANNEL_MESSAGE_WITH_SOURCE = 5
    DEFFERED_UPDATE_MESSAGE = 6
    UPDATE_MESSAGE = 7
    APPLICATION_AUTOCOMPLETE_RESULT = 8
    MODAL = 9


InteractionResponseType = InteractionCallbackType


class MessageComponentType(IntEnum):
    """
        Message component type
    """
    ACTION_ROW = 1
    BUTTON = 2
    SELECT_MENU = 3
    TEXT_INPUT = 4


#############################################################
#   Channel (including messages, embed)
#############################################################


class ChannelType(IntEnum):
    """
        Channel Type
    """
    GUILD_TEXT = 0
    DM = 1
    GUILD_VOICE = 2
    GROUP_DM = 3
    GUILD_CATEGORY = 4
    GUILD_NEWS = 5

    # These three only available in API v9
    GUILD_NEWS_THREAD = 10
    GUILD_PUBLIC_THREAD = 11
    GUILD_PRIVATE_THREAD = 12

    GUILD_STAGE_VOICE = 13
    GUILD_DIRECTORY = 14
    GUILD_FORUM = 15


class VideoQualityMode(IntEnum):
    """
        Video quality mode
    """
    AUTO = 1
    FULL = 2


class ChannelFlag(IntFlag):
    """
        Channel flag
    """
    PINNED = 1 << 1


class AllowedMentionType(str, Enum):
    """
        Allowed mention type
    """
    ROLE = "roles"
    USER = "users"
    EVERYONE = "everyone"


class MessageType(IntEnum):
    """
        Message type
    """
    DEFAULT = 0
    RECIPIENT_ADD = 1
    RECIPIENT_REMOVE = 2
    CALL = 3
    CHANNEL_NAME_CHANGE = 4
    CHANNEL_ICON_CHANGE = 5
    CHANNEL_PINNED_MESSAGE = 6
    GUILD_MEMBER_JOIN = 7
    USER_PREMIUM_GUILD_SUBSCRIPTION = 8
    USER_PREMIUM_GUILD_SUBSCRIPTION_TIER_1 = 9
    USER_PREMIUM_GUILD_SUBSCRIPTION_TIER_2 = 10
    USER_PREMIUM_GUILD_SUBSCRIPTION_TIER_3 = 11
    CHANNEL_FOLLOW_ADD = 12
    GUILD_DISCOVERY_DISQUALIFIED = 14
    GUILD_DISCOVERY_REQUALIFIED = 15
    GUILD_DISCOVERY_GRACE_PERIOD_INITIAL_WARNING = 16
    GUILD_DISCOVERY_GRACE_PERIOD_FINAL_WARNING = 17
    THREAD_CREATED = 18
    REPLY = 19
    CHAT_INPUT_COMMAND = 20
    THREAD_STARTER_MESSAGE = 21
    GUILD_INVITE_REMINDER = 22
    CONTEXT_MENU_COMMAND = 23


class MessageActivityType(IntEnum):
    """
        Message activity type
    """
    JOIN = 1
    SPECTATE = 2
    LISTEN = 3
    JOIN_REQUEST = 5


class MessageFlags(IntFlag):
    """
        Message flags
    """
    CROSSPOSTED = 1 << 0
    IS_CROSSPOST = 1 << 1
    SUPPRESS_EMBEDS = 1 << 2
    SOURCE_MESSAGE_DELETED = 1 << 3
    URGENT = 1 << 4
    HAS_THREAD = 1 << 5
    EPHEMERAL = 1 << 6
    LOADING = 1 << 7
    FAILED_TO_MENTION_SOME_ROLES_IN_THREAD = 1 << 8


class MessageComponentButtonStyle(IntEnum):
    """
        Button style
    """
    PRIMARY = 1
    SECONDARY = 2
    SUCCESS = 3
    DANGER = 4
    LINK = 5


class MessageComponentTextInputStyle(IntEnum):
    """
        Text input style
    """
    SHORT = 1
    PARAGRAPH = 2


class EmbedType(str, Enum):
    """
        Embed content type
    """
    RICH = "rich"
    IMAGE = "image"
    VIDEO = "video"
    GIFV = "gifv"
    ARTICLE = "article"
    LINK = "link"


#############################################################
#   Guild
#############################################################


class DefaultMessageNotificationLevel(IntEnum):
    """
        Default message notification level
    """
    ALL_MESSAGES = 0
    ONLY_MENTIONS = 1


class ExplicitContentFilterLevel(IntEnum):
    """
        Explicit content filter level
    """
    DISABLED = 0
    MEMBERS_WITHOUT_ROLES = 1
    ALL_MEMBERS = 2


class MFALevel(IntEnum):
    """
        Multi-Factor Authentication Level
    """
    NONE = 0
    ELEVATED = 1


class VerificationLevel(IntEnum):
    """
        The verification level a user must comply before joining as a guild member.
    """
    NONE = 0
    LOW = 1
    MEDIUM = 2
    HIGH = 3
    VERY_HIGH = 4


class GuildNSFWLevel(IntEnum):
    """
        Identify how Not-Safe-For-Work (NSFW) a guild is.
    """
    DEFAULT = 0
    EXPLICIT = 1
    SAFE = 2
    AGE_RESTRICTED = 3


class GuildPremiumTier(IntEnum):
    """
        Guild Nitro tier
    """
    NONE = 0
    TIER_1 = 1
    TIER_2 = 2
    TIER_3 = 3


class SystemChannelFlags(IntFlag):
    """
        System channel flags
    """
    SUPPRESS_JOIN_NOTIFICATIONS = 1 << 0
    SUPPRESS_PREMIUM_SUBSCRIPTIONS = 1 << 1
    SUPPRESS_GUILD_REMINDER_NOTIFICATIONS = 1 << 2
    SUPPRESS_JOIN_NOTIFICATION_REPLIES = 1 << 3


class GuildFeature(str, Enum):
    """
        Guild features
    """
    ANIMATED_BANNER = "ANIMATED_BANNER"
    ANIMATED_ICON = "ANIMATED_ICON"
    BANNER = "BANNER"
    COMMERCE = "COMMERCE"
    COMMUNITY = "COMMUNITY"
    DISCOVERABLE = "DISCOVERABLE"
    FEATURABLE = "FEATURABLE"
    INVITE_SPLASH = "INVITE_SPLASH"
    MEMBER_VERIFICATION_GATE_ENABLED = "MEMBER_VERIFICATION_GATE_ENABLED"
    MONETIZATION_ENABLED = "MONETIZATION_ENABLED"
    MORE_STICKERS = "MORE_STICKERS"
    NEWS = "NEWS"
    PARTNERED = "PARTNERED"
    PREVIEW_ENABLED = "PREVIEW_ENABLED"
    PRIVATE_THREADS = "PRIVATE_THREADS"
    ROLE_ICONS = "ROLE_ICONS"
    SEVEN_DAY_THREAD_ARCHIVE = "SEVEN_DAY_THREAD_ARCHIVE"
    THREE_DAY_THREAD_ARCHIVE = "THREE_DAY_THREAD_ARCHIVE"
    TICKETED_EVENTS_ENABLED = "TICKETED_EVENTS_ENABLED"
    VANITY_URL = "VANITY_URL"
    VERIFIED = "VERIFIED"
    VIP_REGIONS = "VIP_REGIONS"
    WELCOME_SCREEN_ENABLED = "WELCOME_SCREEN_ENABLED"


class IntegrationExpireBehavior(IntEnum):
    REMOVE_ROLE = 0
    KICK = 1


#############################################################
#   Guild Scheduled Event
#############################################################


class GuildScheduledEventEntityType(IntEnum):
    """
        Guild scheduled event Entity type
    """
    STAGE_INSTANCE = 1
    VOICE = 2
    EXTERNAL = 3


class GuildScheduledEventStatus(IntEnum):
    """
        Guild scheduled event Status
    """
    SCHEDULED = 1
    ACTIVE = 2
    COMPLERED = 3
    CANCELED = 4


#############################################################
#   Stage Instance
#############################################################


class PrivacyLevel(IntEnum):
    """
        Privacy level
    """
    PUBLIC = 1  # Deprecated
    GUILD_ONLY = 2


GuildScheduledEventPrivacyLevel = PrivacyLevel


#############################################################
#   Invite
#############################################################


class InviteTargetType(IntEnum):
    """
        Invite target type
    """
    STREAM = 1
    EMBEDDED_APPLICATION = 2


#############################################################
#   Presence
#############################################################


class ActivityFlag(IntFlag):
    """
        Activity flags
    """
    INSTANCE = 1 << 0
    JOIN = 1 << 1
    SPECTATE = 1 << 2
    JOIN_REQUEST = 1 << 3
    SYNC = 1 << 4
    PLAY = 1 << 5
    PARTY_PRIVACY_FRIENDS = 1 << 6
    PARTY_PRIVACY_VOICE_CHANNEL = 1 << 7
    EMBEDDED = 1 << 8


#############################################################
#   User
#############################################################


class UserFlag(IntFlag):
    """
        User flag
    """
    STAFF = 1 << 0
    PARTNER = 1 << 1
    HYPESQUAD = 1 << 2
    BUG_HUNTER_LEVEL_1 = 1 << 3
    HYPESQUAD_ONLINE_HOUSE_1 = 1 << 6
    HYPESQUAD_ONLINE_HOUSE_2 = 1 << 7
    HYPESQUAD_ONLINE_HOUSE_3 = 1 << 8
    PREMIUM_EARLY_SUPPORTER = 1 << 9
    TEAM_PSEUDO_USER = 1 << 10
    BUG_HUNTER_LEVEL_2 = 1 << 14
    VERIFIED_BOT = 1 << 16
    VERIFIED_DEVELOPER = 1 << 17
    CERTIFIED_MODERATOR = 1 << 18
    BOT_HTTP_INTERACTIONS = 1 << 19


class PremiumTypes(IntEnum):
    """
        Nitro subscription level
    """
    NONE = 0
    NITRO_CLASSIC = 1
    NITRO = 2


class Visibility(IntEnum):
    """
        Connection visibility
    """
    NONE = 0
    EVERYONE = 1


#############################################################
#   Sticker
#############################################################


class StickerFormatType(IntEnum):
    """
        Sticker format
    """
    PNG = 1
    APNG = 2
    LOTTIE = 3


class StickerType(IntEnum):
    """
        Source of the sticker
    """
    STANDARD = 1
    GUILD = 2


#############################################################
#   Webhook
#############################################################


class WebhookType(IntEnum):
    """
        Webhook type
    """
    INCOMING = 1
    CHANNEL_FOLLOWER = 2
    APPLICATION = 3


#############################################################
#   Activity
#############################################################


class ActivityType(IntEnum):
    """
        User activity presence
    """
    GAME = 0
    STREAMING = 1
    LISTENING = 2
    WATCHING = 3
    CUSTOM = 4
    COMPETING = 5




from dataclasses import dataclass
from typing import Optional, List, Tuple

from . import emoji as emoji_module
from . import discord_types
from . import user as user_module


@dataclass
class ActivityTimestamp(discord_types.DiscordDataClass):
    start: Optional[int] = None
    end: Optional[int] = None


ActivityEmoji = emoji_module.Emoji  # Activity Emoji is essentially a subset of Emoji with the same required properties


@dataclass
class ActivityParty(discord_types.DiscordDataClass):
    id: Optional[str] = None
    size: Optional[Tuple[int, int]] = None  # current_size, max_size


@dataclass
class ActivityAsset(discord_types.DiscordDataClass):
    large_image: Optional[str] = None
    large_text: Optional[str] = None
    small_image: Optional[str] = None
    small_text: Optional[str] = None


@dataclass
class ActivitySecret(discord_types.DiscordDataClass):
    join: Optional[str] = None
    spectate: Optional[str] = None
    match: Optional[str] = None


@dataclass
class ActivityButton(discord_types.DiscordDataClass):
    label: str
    url: str


@dataclass
class Activity(discord_types.DiscordDataClass):
    name: str
    type: discord_types.ActivityType
    created_at: int  # Use unix timestamp here (in milliseconds)
    url: Optional[str] = None
    timestamps: Optional[ActivityTimestamp] = None
    application_id: Optional[discord_types.Snowflake] = None
    details: Optional[str] = None
    state: Optional[str] = None
    emoji: Optional[ActivityEmoji] = None
    party: Optional[ActivityParty] = None
    assets: Optional[ActivityAsset] = None
    secrets: Optional[ActivitySecret] = None
    instance: Optional[bool] = None
    flags: Optional[discord_types.ActivityFlag] = None
    buttons: Optional[List[ActivityButton]] = None

    def __post_init__(self) -> None:
        self.type = discord_types.ActivityType(self.type)
        if isinstance(self.timestamps, dict):
            self.timestamps = ActivityTimestamp(**self.timestamps)  # type: ignore[unreachable]
        if isinstance(self.emoji, dict):
            self.emoji = ActivityEmoji(**self.emoji)  # type: ignore[unreachable]
        if isinstance(self.party, dict):
            self.party = ActivityParty(**self.party)  # type: ignore[unreachable]
        if isinstance(self.assets, dict):
            self.assets = ActivityAsset(**self.assets)  # type: ignore[unreachable]
        if isinstance(self.secrets, dict):
            self.secrets = ActivitySecret(**self.secrets)  # type: ignore[unreachable]
        if isinstance(self.flags, int):
            self.flags = discord_types.ActivityFlag(self.flags)
        if isinstance(self.buttons, list):
            self.buttons = [
                ActivityButton(**b) if isinstance(b, dict) else b
                for b in self.buttons
            ]


@dataclass
class ClientStatus(discord_types.DiscordDataClass):
    desktop: Optional[str] = None
    mobile: Optional[str] = None
    web: Optional[str] = None


@dataclass
class PresenceUpdateEvent(discord_types.DiscordDataClass):
    user: 'user_module.User'
    guild_id: discord_types.Snowflake
    status: str
    activities: List[Activity]
    client_status: ClientStatus

    def __post_init__(self) -> None:
        if isinstance(self.user, dict):
            self.user = user_module.User(**self.user)  # type: ignore[unreachable]
        if isinstance(self.activities, list):
            self.activities = [
                Activity(**a) if isinstance(a, dict) else a
                for a in self.activities
            ]
        if isinstance(self.client_status, dict):
            self.client_status = ClientStatus(**self.client_status)  # type: ignore[unreachable]




from dataclasses import dataclass
from typing import List, Optional, Any, Union
from typing_extensions import Self  # type: ignore[attr-defined]

from . import guild
from . import discord_types
from . import user as user_module
from . import emoji as emoji_module
from . import channel
from . import sticker
from . import application as application_module


@dataclass
class AllowedMentions(discord_types.DiscordDataClass):
    parse: List[discord_types.AllowedMentionType]
    roles: List[discord_types.Snowflake]
    users: List[discord_types.Snowflake]
    replied_user: bool

    def __post_init__(self) -> None:
        self.parse = [
            discord_types.AllowedMentionType(t) if isinstance(t, str) else t
            for t in self.parse
        ]


@dataclass
class ThreadMetadata(discord_types.DiscordDataClass):
    archived: bool
    auto_archive_duration: int
    archive_timestamp: str
    locked: bool
    invitable: Optional[bool] = None
    create_timestamp: Optional[str] = None


@dataclass
class ThreadMember(discord_types.DiscordDataClass):
    join_timestamp: str
    flags: int
    id: Optional[discord_types.Snowflake] = None
    user_id: Optional[discord_types.Snowflake] = None


@dataclass
class Overwrite(discord_types.DiscordDataClass):
    id: discord_types.Snowflake
    type: discord_types.Snowflake
    allow: str
    deny: str


@dataclass
class Channel(discord_types.DiscordDataClass):
    id: discord_types.Snowflake
    type: discord_types.ChannelType
    guild_id: Optional[discord_types.Snowflake] = None
    position: Optional[int] = None
    permission_overwrites: Optional[List[Overwrite]] = None
    name: Optional[str] = None
    topic: Optional[str] = None
    nsfw: Optional[bool] = None
    last_message_id: Optional[discord_types.Snowflake] = None
    bitrate: Optional[int] = None
    user_limit: Optional[int] = None
    rate_limit_per_user: Optional[int] = None
    recipients: Optional[List['user_module.User']] = None
    icon: Optional[str] = None
    owner_id: Optional[discord_types.Snowflake] = None
    application_id: Optional[discord_types.Snowflake] = None
    parent_id: Optional[discord_types.Snowflake] = None
    last_pin_timestamp: Optional[str] = None
    rtc_region: Optional[str] = None
    video_quality_mode: Optional[discord_types.VideoQualityMode] = None
    message_count: Optional[int] = None
    member_count: Optional[int] = None
    thread_metadata: Optional[ThreadMetadata] = None
    member: Optional[ThreadMember] = None
    default_auto_archive_duration: Optional[int] = None
    permissions: Optional[str] = None
    flags: Optional[discord_types.ChannelFlag] = None

    def __post_init__(self) -> None:
        if isinstance(self.recipients, list):
            self.recipients = [user_module.User(**u) if isinstance(u, dict) else u for u in self.recipients]
        if isinstance(self.video_quality_mode, int):
            self.video_quality_mode = discord_types.VideoQualityMode(self.video_quality_mode)
        if isinstance(self.flags, int):
            self.flags = discord_types.ChannelFlag(self.flags)
        if isinstance(self.thread_metadata, dict):
            self.thread_metadata = ThreadMetadata(**self.thread_metadata)  # type: ignore[unreachable]
        if isinstance(self.member, dict):
            self.member = ThreadMember(**self.member)  # type: ignore[unreachable]
        if isinstance(self.permission_overwrites, list):
            self.permission_overwrites = [
                Overwrite(**overwrite) if isinstance(overwrite, dict) else overwrite
                for overwrite in self.permission_overwrites
            ]


PartialChannel = Channel


@dataclass
class ChannelMention(discord_types.DiscordDataClass):
    id: discord_types.Snowflake
    guild_id: discord_types.Snowflake
    type: discord_types.ChannelType
    name: str

    def __post_init__(self) -> None:
        self.type = discord_types.ChannelType(self.type)


@dataclass
class EmbedThumbnail(discord_types.DiscordDataClass):
    url: str
    proxy_url: Optional[str] = None
    height: Optional[int] = None
    width: Optional[int] = None


@dataclass
class EmbedVideo(discord_types.DiscordDataClass):
    url: Optional[str] = None
    proxy_url: Optional[str] = None
    height: Optional[int] = None
    width: Optional[int] = None


@dataclass
class EmbedImage(discord_types.DiscordDataClass):
    url: str
    proxy_url: Optional[str] = None
    height: Optional[int] = None
    width: Optional[int] = None


@dataclass
class EmbedProvider(discord_types.DiscordDataClass):
    name: Optional[str] = None
    url: Optional[str] = None


@dataclass
class EmbedAuthor(discord_types.DiscordDataClass):
    name: str
    url: Optional[str] = None
    icon_url: Optional[str] = None
    proxy_icon_url: Optional[str] = None


@dataclass
class EmbedFooter(discord_types.DiscordDataClass):
    text: str
    icon_url: Optional[str] = None
    proxy_icon_url: Optional[str] = None


@dataclass
class EmbedField(discord_types.DiscordDataClass):
    name: str
    value: str
    inline: Optional[bool] = None


@dataclass
class Embed(discord_types.DiscordDataClass):
    title: Optional[str] = None
    type: Optional[discord_types.EmbedType] = None
    description: Optional[str] = None
    url: Optional[str] = None
    timestamp: Optional[str] = None
    color: Optional[int] = None
    footer: Optional[EmbedFooter] = None
    image: Optional[EmbedImage] = None
    thumbnail: Optional[EmbedThumbnail] = None
    video: Optional[EmbedVideo] = None
    provider: Optional[EmbedProvider] = None
    author: Optional[EmbedAuthor] = None
    fields: Optional[List[EmbedField]] = None

    def __post_init__(self) -> None:
        if isinstance(self.type, str):
            self.type = discord_types.EmbedType(self.type)
        if isinstance(self.footer, dict):
            self.footer = EmbedFooter(**self.footer)  # type: ignore[unreachable]
        if isinstance(self.image, dict):
            self.image = EmbedImage(**self.image)  # type: ignore[unreachable]
        if isinstance(self.thumbnail, dict):
            self.thumbnail = EmbedThumbnail(**self.thumbnail)  # type: ignore[unreachable]
        if isinstance(self.video, dict):
            self.video = EmbedVideo(**self.video)  # type: ignore[unreachable]
        if isinstance(self.provider, dict):
            self.provider = EmbedProvider(**self.provider)  # type: ignore[unreachable]
        if isinstance(self.author, dict):
            self.author = EmbedAuthor(**self.author)  # type: ignore[unreachable]
        if isinstance(self.fields, list):
            self.fields = [
                EmbedField(**field) if isinstance(field, dict) else field
                for field in self.fields
            ]


@dataclass
class PartialAttachment(discord_types.DiscordDataClass):
    id: discord_types.Snowflake
    filename: str
    description: Optional[str] = None
    content_type: Optional[str] = None


@dataclass
class Attachment(discord_types.DiscordDataClass):
    id: discord_types.Snowflake
    filename: str
    size: int
    url: str
    proxy_url: str
    description: Optional[str] = None
    content_type: Optional[str] = None
    height: Optional[int] = None
    width: Optional[int] = None
    ephemeral: Optional[bool] = None


@dataclass
class MessageComponent(discord_types.DiscordDataClass):
    def __new__(cls, *args: Any, **kwargs: Any) -> Union[  # type: ignore[misc]
        'MessageComponentActionRow',
        'MessageComponentButton',
        'MessageComponentSelectMenu',
        'MessageComponentTextInput'
    ]:
        CLASS_MAP = {
            discord_types.MessageComponentType.ACTION_ROW: MessageComponentActionRow,
            discord_types.MessageComponentType.BUTTON: MessageComponentButton,
            discord_types.MessageComponentType.SELECT_MENU: MessageComponentSelectMenu,
            discord_types.MessageComponentType.TEXT_INPUT: MessageComponentTextInput
        }
        target = CLASS_MAP[kwargs['type']]
        if cls is MessageComponent:
            return super(MessageComponent, target).__new__(target)  # type: ignore[misc, no-any-return]
        else:
            return super(MessageComponent, cls).__new__(cls, *args, **kwargs)  # type: ignore[return-value]


@dataclass
class MessageComponentActionRow(MessageComponent):
    components: List[MessageComponent]
    type: discord_types.MessageComponentType = discord_types.MessageComponentType.ACTION_ROW

    def __post_init__(self) -> None:
        self.components = [
            MessageComponent(component) if isinstance(component, dict) else component
            for component in self.components
        ]
        if self.type != discord_types.MessageComponentType.ACTION_ROW:
            raise ValueError(f"type for {self.__class__.__name__} must be MessageComponentType.ACTION_ROW")
        self.type = discord_types.MessageComponentType.ACTION_ROW


@dataclass
class MessageComponentButton(MessageComponent):
    style: discord_types.MessageComponentButtonStyle
    type: discord_types.MessageComponentType = discord_types.MessageComponentType.BUTTON
    label: Optional[str] = None
    emoji: Optional[emoji_module.Emoji] = None
    custom_id: Optional[str] = None
    url: Optional[str] = None
    disabled: Optional[bool] = None

    def __post_init__(self) -> None:
        self.style = discord_types.MessageComponentButtonStyle(self.style)
        if self.type != discord_types.MessageComponentType.BUTTON:
            raise ValueError(f"type for {self.__class__.__name__} must be MessageComponentType.BUTTON")
        self.type = discord_types.MessageComponentType.BUTTON
        if isinstance(self.emoji, dict):
            self.emoji = emoji_module.Emoji(**self.emoji)  # type: ignore[unreachable]


@dataclass
class MessageComponentSelectOption(discord_types.DiscordDataClass):
    label: str
    value: str
    default: bool = False
    description: Optional[str] = None
    emoji: Optional[emoji_module.Emoji] = None

    def __post_init__(self) -> None:
        if isinstance(self.emoji, dict):
            self.emoji = emoji_module.Emoji(**self.emoji)  # type: ignore[unreachable]


@dataclass
class MessageComponentSelectMenu(MessageComponent):
    custom_id: str
    options: List[MessageComponentSelectOption]
    min_values: int = 1
    max_values: int = 1
    type: discord_types.MessageComponentType = discord_types.MessageComponentType.SELECT_MENU
    disabled: bool = False
    placeholder: Optional[str] = None

    def __post_init__(self) -> None:
        self.options = [
            MessageComponentSelectOption(**option) if isinstance(option, dict) else option
            for option in self.options
        ]
        if self.type != discord_types.MessageComponentType.SELECT_MENU:
            raise ValueError(f"type for {self.__class__.__name__} must be MessageComponentType.SELECT_MENU")
        self.type = discord_types.MessageComponentType.SELECT_MENU


@dataclass
class MessageComponentTextInput(MessageComponent):
    custom_id: str
    style: discord_types.MessageComponentTextInputStyle
    label: str
    type: discord_types.MessageComponentType = discord_types.MessageComponentType.TEXT_INPUT
    min_length: Optional[int] = None
    max_length: Optional[int] = None
    required: bool = True
    value: Optional[str] = None
    placeholder: Optional[str] = None

    def __post_init__(self) -> None:
        self.style = discord_types.MessageComponentTextInputStyle(self.style)
        if self.type != discord_types.MessageComponentType.TEXT_INPUT:
            raise ValueError(f"type for {self.__class__.__name__} must be MessageComponentType.TEXT_INPUT")
        self.type = discord_types.MessageComponentType.TEXT_INPUT


@dataclass
class MessageReference(discord_types.DiscordDataClass):
    message_id: Optional[discord_types.Snowflake] = None
    channel_id: Optional[discord_types.Snowflake] = None
    guild_id: Optional[discord_types.Snowflake] = None
    fail_if_not_exists: Optional[bool] = True


@dataclass
class MessageInteraction(discord_types.DiscordDataClass):
    id: discord_types.Snowflake
    type: discord_types.InteractionType
    name: str
    user: 'user_module.User'
    member: Optional['guild.PartialGuildMember'] = None

    def __post_init__(self) -> None:
        self.type = discord_types.InteractionType(self.type)
        if isinstance(self.user, dict):
            self.user = user_module.User(**self.user)  # type: ignore[unreachable]
        if isinstance(self.member, dict):
            self.member = guild.PartialGuildMember(**self.member)  # type: ignore[unreachable]


@dataclass
class Reaction(discord_types.DiscordDataClass):
    count: int
    me: bool
    emoji: emoji_module.Emoji


@dataclass
class MessageActivity(discord_types.DiscordDataClass):
    type: discord_types.MessageActivityType
    party_id: Optional[str] = None


@dataclass
class Message(discord_types.DiscordDataClass):
    id: discord_types.Snowflake
    channel_id: discord_types.Snowflake
    timestamp: str
    tts: bool
    mention_everyone: bool
    mentions: List['user_module.User']
    mention_roles: List['guild.Role']
    attachments: List[Attachment]
    embeds: List[Embed]
    content: str
    type: discord_types.MessageType
    guild_id: Optional[discord_types.Snowflake] = None
    author: Optional['user_module.User'] = None
    member: Optional['guild.PartialGuildMember'] = None
    edited_timestamp: Optional[str] = None
    mention_channels: Optional[List[channel.ChannelMention]] = None
    reactions: Optional[List[Reaction]] = None
    nonce: Optional[Union[int, str]] = None
    activity: Optional[MessageActivity] = None
    application: Optional['application_module.PartialApplication'] = None
    application_id: Optional[discord_types.Snowflake] = None
    message_reference: Optional[MessageReference] = None
    flags: Optional[discord_types.MessageFlags] = None
    # Make mypy ignore "Self if not a type" error
    referenced_message: Optional[Self] = None  # type: ignore
    interaction: Optional[MessageInteraction] = None
    thread: Optional[channel.Channel] = None
    components: Optional[List[MessageComponent]] = None
    sticker_items: Optional[List[sticker.StickerItem]] = None
    stickers: Optional[List[sticker.Sticker]] = None

    def __post_init__(self) -> None:
        self.mentions = [
            user_module.User(**u) if isinstance(u, dict) else u
            for u in self.mentions
        ]
        self.mention_roles = [
            guild.Role(**role) if isinstance(role, dict) else role
            for role in self.mention_roles
        ]
        self.attachments = [
            Attachment(**attachment) if isinstance(attachment, dict) else attachment
            for attachment in self.attachments
        ]
        self.embeds = [
            Embed(**embed) if isinstance(embed, dict) else embed
            for embed in self.embeds
        ]
        self.type = discord_types.MessageType(self.type)
        if isinstance(self.author, dict):
            self.author = user_module.User(**self.author)  # type: ignore[unreachable]
        if isinstance(self.member, dict):
            self.member = guild.PartialGuildMember(**self.member)  # type: ignore[unreachable]
        if isinstance(self.mention_channels, list):
            self.mention_channels = [
                channel.ChannelMention(**cm) if isinstance(cm, dict) else cm
                for cm in self.mention_channels
            ]
        if isinstance(self.reactions, list):
            self.reactions = [
                Reaction(**reaction) if isinstance(reaction, dict) else reaction
                for reaction in self.reactions
            ]
        if isinstance(self.flags, int):
            self.flags = discord_types.MessageFlags(self.flags)
        if isinstance(self.message_reference, dict):
            self.message_reference = MessageReference(**self.message_reference)  # type: ignore[unreachable]
        if isinstance(self.referenced_message, dict):
            self.referenced_message = Message(**self.referenced_message)  # type: ignore[unreachable]
        if isinstance(self.interaction, dict):
            self.interaction = MessageInteraction(**self.interaction)  # type: ignore[unreachable]
        if isinstance(self.thread, dict):
            self.thread = channel.Channel(**self.thread)  # type: ignore[unreachable]
        if isinstance(self.components, list):
            self.components = [
                MessageComponent(**component) if isinstance(component, dict) else component
                for component in self.components
            ]
        if isinstance(self.activity, dict):
            self.activity = MessageActivity(**self.activity)  # type: ignore[unreachable]
        if isinstance(self.sticker_items, list):
            self.sticker_items = [
                sticker.StickerItem(**item) if isinstance(item, dict) else item
                for item in self.sticker_items
            ]
        if isinstance(self.stickers, list):
            self.stickers = [
                sticker.Sticker(**stkr) if isinstance(stkr, dict) else stkr
                for stkr in self.stickers  # name as stkr due to imported mudule name also called sticker
            ]


@dataclass
class FollowedChannel(discord_types.DiscordDataClass):
    channel_id: discord_types.Snowflake
    webhook_id: discord_types.Snowflake




from dataclasses import dataclass
from typing import Optional

from . import discord_types
from . import user as user_module
from . import guild as guild_module


@dataclass
class GuildScheduledEventEntityMetadata(discord_types.DiscordDataClass):
    location: Optional[str] = None


@dataclass
class GuildScheduledEvent(discord_types.DiscordDataClass):
    id: discord_types.Snowflake
    guild_id: discord_types.Snowflake
    name: str
    scheduled_start_time: str
    privacy_level: discord_types.GuildScheduledEventPrivacyLevel
    status: discord_types.GuildScheduledEventStatus
    entity_type: discord_types.GuildScheduledEventEntityType
    creator_id: Optional[discord_types.Snowflake] = None
    description: Optional[str] = None
    creator: Optional['user_module.User'] = None
    user_count: Optional[int] = None
    image: Optional[str] = None

    # Required for STAGE_INSTANCE and VOICE type
    channel_id: Optional[discord_types.Snowflake] = None

    # Required for EXTERNAL type
    entity_metadata: Optional[GuildScheduledEventEntityMetadata] = None
    scheduled_end_time: Optional[str] = None

    def __post_init__(self) -> None:
        self.privacy_level = discord_types.GuildScheduledEventPrivacyLevel(self.privacy_level)
        self.status = discord_types.GuildScheduledEventStatus(self.status)
        self.entity_type = discord_types.GuildScheduledEventEntityType(self.entity_type)
        if isinstance(self.creator, dict):
            self.creator = user_module.User(**self.creator)  # type: ignore[unreachable]

        if self.entity_type is discord_types.GuildScheduledEventEntityType.EXTERNAL:
            if self.entity_metadata is None or self.scheduled_end_time is None:
                raise AttributeError("entity_metadata and scheduled_end_time is required for EXTERNAL event type.")
            if isinstance(self.entity_metadata, dict):
                self.entity_metadata = GuildScheduledEventEntityMetadata(**self.entity_metadata)  # type: ignore[unreachable]
        else:
            if self.channel_id is None:
                raise AttributeError(f"entity_metadata and scheduled_end_time is required for {self.entity_type.name} event type.")


@dataclass
class GuildScheduledEventUser(discord_types.DiscordDataClass):
    guild_scheduled_event_id: discord_types.Snowflake
    user: 'user_module.User'
    member: Optional['guild_module.GuildMember']

    def __post_init__(self) -> None:
        if isinstance(self.user, dict):
            self.user = user_module.User(**self.user)  # type: ignore[unreachable]
        if isinstance(self.member, dict):
            self.member = guild_module.GuildMember(**self.member)  # type: ignore[unreachable]




from dataclasses import dataclass
from typing import Dict, List, Optional, Union, Any
from typing_extensions import Self  # type: ignore[attr-defined]

from . import discord_types
from . import channel
from . import user as user_module
from . import guild as guild_module
from . import application


@dataclass
class ApplicationCommandOptionChoice(discord_types.DiscordDataClass):
    """
        Related choice for parameter (ApplicationCommandOption)
    """
    name: str
    value: Union[str, int, float]
    name_localizations: Optional[Dict[str, str]] = None


@dataclass
class ApplicationCommandOption(discord_types.DiscordDataClass):
    """
        Parameter for application command.
    """
    name: str
    description: str
    type: discord_types.ApplicationCommandOptionType = discord_types.ApplicationCommandOptionType.SUB_COMMAND
    name_localizations: Optional[Dict[str, str]] = None
    description_localizations: Optional[Dict[str, str]] = None
    required: Optional[bool] = False
    choices: Optional[List['ApplicationCommandOptionChoice']] = None
    options: Optional[List['ApplicationCommandOption']] = None
    channel_types: Optional[List[discord_types.ChannelType]] = None
    min_value: Optional[Union[int, float]] = None
    max_value: Optional[Union[int, float]] = None
    autocomplete: Optional[bool] = False

    def __post_init__(self) -> None:
        self.type = discord_types.ApplicationCommandOptionType(self.type)
        if isinstance(self.choices, list):
            self.choices = [ApplicationCommandOptionChoice(**choice) if isinstance(choice, dict) else choice for choice in self.choices]
        if isinstance(self.options, list):
            self.options = [ApplicationCommandOption(**option) if isinstance(option, dict) else option for option in self.options]
        if isinstance(self.channel_types, list):
            self.channel_types = [
                discord_types.ChannelType(channel_type) if isinstance(channel_type, discord_types.ChannelType) else channel_type
                for channel_type in self.channel_types
            ]


@dataclass
class ApplicationCommand(discord_types.DiscordDataClass):
    """
        The specification for the custom Discord command.
    """
    name: str
    description: str
    type: discord_types.ApplicationCommandType = discord_types.ApplicationCommandType.CHAT_INPUT
    name_localizations: Optional[Dict[str, str]] = None
    description_localizations: Optional[Dict[str, str]] = None
    id: Optional[discord_types.Snowflake] = None
    application_id: Optional[discord_types.Snowflake] = None
    guild_id: Optional[discord_types.Snowflake] = None
    options: Optional[List['ApplicationCommandOption']] = None
    default_permission: Optional[bool] = None
    version: Optional[discord_types.Snowflake] = None

    # The following attribute are undocumented
    default_member_permissions: Optional[Any] = None
    dm_permission: Optional[Any] = None

    def __post_init__(self) -> None:
        self.type = discord_types.ApplicationCommandType(self.type)
        if isinstance(self.options, list):
            self.options = [ApplicationCommandOption(**option) if isinstance(option, dict) else option for option in self.options]


@dataclass
class InteractionResponseData(discord_types.DiscordDataClass):
    def __new__(cls, *args: Any, **kwargs: Any) -> Union[  # type: ignore[misc]
        'InteractionResponseMessage',
        'InteractionResponseAutocomplete',
        'InteractionResponseModal'
    ]:
        target: Any = InteractionResponseMessage
        if "choices" in kwargs:
            target = InteractionResponseAutocomplete
        if "title" in kwargs or "custom_id" in kwargs:
            target = InteractionResponseModal
        if cls is InteractionResponseData:
            return super(InteractionResponseData, target).__new__(target, *args, **kwargs)  # type: ignore[call-arg, no-any-return]
        else:
            return super(InteractionResponseData, cls).__new__(cls)  # type: ignore[return-value]


@dataclass
class InteractionResponseMessage(InteractionResponseData):
    tts: Optional[bool] = None
    content: Optional[str] = None
    embeds: Optional[List[channel.Embed]] = None
    allowed_mentions: Optional[channel.AllowedMentions] = None
    flags: Optional[discord_types.MessageFlags] = None
    components: Optional[List[channel.MessageComponent]] = None
    attachments: Optional[List[channel.PartialAttachment]] = None

    def __post_init__(self) -> None:
        if isinstance(self.embeds, list):
            self.embeds = [
                channel.Embed(**e) if isinstance(e, dict) else e
                for e in self.embeds
            ]
        if isinstance(self.allowed_mentions, dict):
            self.allowed_mentions = channel.AllowedMentions(**self.allowed_mentions)  # type: ignore[unreachable]
        if isinstance(self.flags, int):
            self.flags = discord_types.MessageFlags(self.flags)
        if isinstance(self.components, list):
            self.components = [
                channel.MessageComponent(**component) if isinstance(component, dict) else component
                for component in self.components
            ]
        if isinstance(self.attachments, list):
            self.attachments = [
                channel.PartialAttachment(**attachment) if isinstance(attachment, dict) else attachment
                for attachment in self.attachments
            ]


@dataclass
class InteractionResponseAutocomplete(InteractionResponseData):
    choices: List[ApplicationCommandOptionChoice]

    def __post_init__(self) -> None:
        self.choices = [
            ApplicationCommandOptionChoice(**choice) if isinstance(choice, dict) else choice
            for choice in self.choices
        ]


@dataclass
class InteractionResponseModal(InteractionResponseData):
    custom_id: str
    title: str
    components: List[channel.MessageComponent]

    def __post_init__(self) -> None:
        self.components = [
            channel.MessageComponent(**component) if isinstance(component, dict) else component
            for component in self.components
        ]


@dataclass
class InteractionResponse(discord_types.DiscordDataClass):
    """
        Response object for handling incoming interaction.
    """
    type: discord_types.InteractionResponseType
    data: Optional[InteractionResponseData] = None

    def __post_init__(self) -> None:
        self.type = discord_types.InteractionResponseType(self.type)
        if isinstance(self.data, dict):
            # TODO: The corresponding data may related to the response type. But I'm not sure if deciding data type here is a good idea.
            DATA_TYPES = {  # type: ignore[unreachable]
                discord_types.InteractionResponseType.CHANNEL_MESSAGE_WITH_SOURCE: InteractionResponseMessage,
                discord_types.InteractionResponseType.DEFERRED_CHANNEL_MESSAGE_WITH_SOURCE: InteractionResponseMessage,
                discord_types.InteractionResponseType.DEFFERED_UPDATE_MESSAGE: InteractionResponseMessage,
                discord_types.InteractionResponseType.UPDATE_MESSAGE: InteractionResponseMessage,
                discord_types.InteractionResponseType.MODAL: InteractionResponseModal,
                discord_types.InteractionResponseType.APPLICATION_AUTOCOMPLETE_RESULT: InteractionResponseAutocomplete,
                discord_types.InteractionResponseType.PONG: None
            }
            target_cls = DATA_TYPES[self.type]
            if target_cls is None:
                # Response to ping shouldn't have data
                self.data = None
            else:
                self.data = target_cls(**self.data)


@dataclass
class InteractionResolvedData(discord_types.DiscordDataClass):
    """
        Interaction Resolve Data
    """
    users: Optional[Dict[discord_types.Snowflake, user_module.User]] = None
    members: Optional[Dict[discord_types.Snowflake, guild_module.PartialGuildMember]] = None
    roles: Optional[Dict[discord_types.Snowflake, guild_module.Role]] = None
    channels: Optional[Dict[discord_types.Snowflake, channel.PartialChannel]] = None
    messages: Optional[Dict[discord_types.Snowflake, channel.Message]] = None
    attachments: Optional[Dict[discord_types.Snowflake, channel.Attachment]] = None

    def __post_init__(self) -> None:
        if isinstance(self.users, dict):
            self.users = {
                discord_types.Snowflake(k): user_module.User(**v) if isinstance(v, dict) else v
                for (k, v) in self.users.items()
            }
        if isinstance(self.members, dict):
            self.members = {
                discord_types.Snowflake(k): guild_module.PartialGuildMember(**v) if isinstance(v, dict) else v
                for (k, v) in self.members.items()
            }
        if isinstance(self.roles, dict):
            self.roles = {
                discord_types.Snowflake(k): guild_module.Role(**v) if isinstance(v, dict) else v
                for (k, v) in self.roles.items()
            }
        if isinstance(self.channels, dict):
            self.channels = {
                discord_types.Snowflake(k): channel.Channel(**v) if isinstance(v, dict) else v
                for (k, v) in self.channels.items()
            }
        if isinstance(self.messages, dict):
            self.messages = {
                discord_types.Snowflake(k): channel.Message(**v) if isinstance(v, dict) else v
                for (k, v) in self.messages.items()
            }
        if isinstance(self.attachments, dict):
            self.attachments = {
                discord_types.Snowflake(k): channel.Attachment(**v) if isinstance(v, dict) else v
                for (k, v) in self.attachments.items()
            }


@dataclass
class InteractionDataOption(discord_types.DiscordDataClass):
    """
        When incoming interaction message is application command,
        this object is to deliver the command parameter.
    """
    name: str
    type: discord_types.ApplicationCommandOptionType
    value: Optional[Union[int, float, str]] = None
    # Make mypy ignore "Self if not a type" error
    options: Optional[List[Self]] = None  # type: ignore
    focused: Optional[bool] = None

    def __post_init__(self) -> None:
        self.type = discord_types.ApplicationCommandOptionType(self.type)
        if isinstance(self.options, list):
            self.options = [
                InteractionDataOption(**option) if isinstance(option, dict) else option
                for option in self.options
            ]


@dataclass
class InteractionData(discord_types.DiscordDataClass):
    """
        Additional data for incoming interaction message
    """
    id: discord_types.Snowflake
    name: str
    type: discord_types.ApplicationCommandType
    resolved: Optional[InteractionResolvedData] = None
    options: Optional[List[InteractionDataOption]] = None
    guild_id: Optional[discord_types.Snowflake] = None
    custom_id: Optional[str] = None
    component_type: Optional[discord_types.MessageComponentType] = None
    values: Optional[List[channel.MessageComponentSelectOption]] = None
    target_id: Optional[discord_types.Snowflake] = None
    components: Optional[List[channel.MessageComponent]] = None

    def __post_init__(self) -> None:
        self.type = discord_types.ApplicationCommandType(self.type)
        if isinstance(self.resolved, dict):
            self.resolved = InteractionResolvedData(**self.resolved)  # type: ignore[unreachable]
        if isinstance(self.options, list):
            self.options = [
                InteractionDataOption(**option) if isinstance(option, dict) else option
                for option in self.options
            ]
        if isinstance(self.component_type, int):
            self.component_type = discord_types.MessageComponentType(self.component_type)
        if isinstance(self.values, list):
            self.values = [
                channel.MessageComponentSelectOption(**option) if isinstance(option, dict) else option
                for option in self.values
            ]
        if isinstance(self.components, list):
            self.components = [
                channel.MessageComponent(**component) if isinstance(component, dict) else component
                for component in self.components
            ]


@dataclass
class InteractionRequest(discord_types.DiscordDataClass):
    """
        Incoming interaction message
    """
    id: discord_types.Snowflake
    application_id: discord_types.Snowflake
    type: discord_types.InteractionType
    token: str
    version: int = 1
    data: Optional[InteractionData] = None
    guild_id: Optional[discord_types.Snowflake] = None
    channel_id: Optional[discord_types.Snowflake] = None
    member: Optional[guild_module.GuildMember] = None
    user: Optional[user_module.User] = None
    message: Optional[channel.Message] = None
    locale: Optional[str] = None
    guild_locale: Optional[str] = None

    # Internal use only
    _app: Optional['application.Application'] = None

    def __post_init__(self) -> None:
        self.type = discord_types.InteractionType(self.type)
        if isinstance(self.data, dict):  # type: ignore[unreachable]
            self.data = InteractionData(**self.data)  # type: ignore[unreachable]
        if isinstance(self.member, dict):
            self.member = guild_module.GuildMember(**self.member)  # type: ignore[unreachable]
        if isinstance(self.user, dict):
            self.user = user_module.User(**self.user)  # type: ignore[unreachable]
        if isinstance(self.message, dict):
            self.message = channel.Message(**self.message)  # type: ignore[unreachable]

    def get_channel(self) -> Optional[channel.Channel]:
        if self.channel_id and self._app:
            channel_data, _ = self._app.call_api(  # type: ignore[misc]
                "GET",
                f"/channels/{self.channel_id}"
            )
            return channel.Channel(**channel_data)
        else:
            return None




from typing import Optional
from dataclasses import dataclass

from . import discord_types
from . import guild as guild_module


@dataclass
class VoiceState(discord_types.DiscordDataClass):
    channel_id: discord_types.Snowflake
    user_id: discord_types.Snowflake
    session_id: str
    deaf: bool
    mute: bool
    self_deaf: bool
    self_mute: bool
    self_video: bool
    suppress: bool
    request_to_speak_timestamp: str
    guild_id: Optional[discord_types.Snowflake] = None
    member: Optional['guild_module.GuildMember'] = None
    self_stream: Optional[bool] = None

    def __post_init__(self) -> None:
        if isinstance(self.member, dict):
            self.member = guild_module.GuildMember(**self.member)  # type: ignore[unreachable]


PartialVoiceState = VoiceState




from .application import\
    Application

from .discord_types import\
    ApplicationCommandType,\
    ApplicationCommandOptionType,\
    InteractionResponseType,\
    InteractionType

from .interaction import \
    InteractionResponse,\
    InteractionResponseData,\
    InteractionRequest,\
    InteractionResponseAutocomplete,\
    InteractionResponseModal,\
    InteractionResponseMessage,\
    ApplicationCommand,\
    ApplicationCommandOption,\
    ApplicationCommandOptionChoice



"""
    Function wrapper for discord application interaction concept using Flask.
"""

import json
from typing import Any, Callable, Dict, List, Optional, Tuple, Union
from dataclasses import asdict, dataclass, field
import logging
from typing_extensions import Self  # type: ignore[attr-defined]
import requests
import requests.structures
import flask
# Added "type: ignore" so mypy won't complain nacl doesn't have stub or py.typed.
import nacl.signing     # type: ignore
import nacl.encoding    # type: ignore
import nacl.exceptions  # type: ignore

from discord_app import user

from . import discord_types
from . import interaction


def asdict_ignore_none(x: List[Tuple[str, Any]]) -> Dict[str, Any]:
    def condition(k: str, v: Any) -> bool:
        return (v is not None) and\
            (v) and \
            k[0] != "_"
    return {k: v for (k, v) in x if condition(k, v)}


def _flask_response_from_interaction(
    response: interaction.InteractionResponse,
    headers: Dict[str, str] = {},
    status: int = 200
) -> flask.Response:
    return flask.Response(
        response=json.dumps(asdict(response, dict_factory=asdict_ignore_none)),
        headers=headers,
        status=status,
        mimetype="application/json"
    )


@dataclass
class InstallParams(discord_types.DiscordDataClass):
    scopes: List[str]
    permissions: str


@dataclass
class Application(discord_types.DiscordDataClass):
    """
        Wrapper class for Discord Application Interaction model.
    """
    id: discord_types.Snowflake
    name: str
    description: str
    bot_public: bool
    bot_require_code_grant: bool
    verify_key: str
    team: Optional[Any] = None
    icon: Optional[str] = None
    rpc_origins: Optional[List[str]] = None
    terms_of_service_url: Optional[str] = None
    privacy_policy_url: Optional[str] = None
    owner: Optional[user.User] = None
    guild_id: Optional[discord_types.Snowflake] = None
    primary_sku_id: Optional[discord_types.Snowflake] = None
    slug: Optional[str] = None
    cover_image: Optional[str] = None
    flags: Optional[discord_types.ApplicationFlags] = None
    tags: Optional[List[str]] = None
    install_params: Optional[InstallParams] = None
    custom_install_url: Optional[str] = None

    # These attributes are undocumented for some reason
    summary: Optional[Any] = None
    type: Optional[Any] = None
    hook: Optional[bool] = None

    _public_key: Optional[str] = None
    _client_secret: Optional[str] = None
    _bot_token: Optional[str] = None
    _endpoint: Optional[str] = None
    _command_list: Optional[Dict[str, Any]] = field(default_factory=dict)  # type: ignore[assignment]
    _logger: Optional[logging.Logger] = None
    _previously_registered_commands: Optional[List[Dict[str, Any]]] = None

    def __post_init__(self) -> None:
        if isinstance(self.owner, dict):
            self.owner = user.User(**self.owner)  # type: ignore[unreachable]
        if isinstance(self.flags, int):
            self.flags = discord_types.ApplicationFlags(self.flags)
        if isinstance(self.install_params, dict):
            self.install_params = InstallParams(**self.install_params)  # type: ignore[unreachable]

        if self._public_key is not None and self._bot_token is not None:
            # Initializing flask
            self._flask: flask.Flask = flask.Flask(self.name)
            self._logger: logging.Logger = self._flask.logger if self._logger is None else self._logger

            def _handle_command() -> flask.Response:
                """
                    Handler for incoming message.
                """
                # Request must is application/json
                if not flask.request.is_json:
                    self._logger.warning("Invalid request: Content-Type is not application/json")  # type: ignore[union-attr]
                    return _flask_response_from_interaction(
                        interaction.InteractionResponse(
                            type=discord_types.InteractionResponseType.CHANNEL_MESSAGE_WITH_SOURCE,
                            data=interaction.InteractionResponseMessage(
                                content="Invalid message: Request content type must be json"
                            )
                        ),
                        status=400
                    )

                # Request signature verification
                verifier = nacl.signing.VerifyKey(self._public_key.encode("ascii"), nacl.encoding.HexEncoder)  # type: ignore[union-attr]
                verified = False
                try:
                    verifier.verify(
                        flask.request.headers["X-Signature-Timestamp"].encode("utf-8") + flask.request.data,
                        nacl.encoding.HexEncoder.decode(flask.request.headers["X-Signature-Ed25519"].encode("ascii"))
                    )
                    verified = True
                except nacl.exceptions.BadSignatureError:
                    return _flask_response_from_interaction(
                        interaction.InteractionResponse(
                            type=discord_types.InteractionResponseType.CHANNEL_MESSAGE_WITH_SOURCE,
                            data=interaction.InteractionResponseMessage(
                                content="Invalid message: Failed to test signature"
                            )
                        ),
                        status=401
                    )
                if not verified:
                    self._logger.info("Message verification failed.")  # type: ignore[union-attr]
                    # Mandated 401 response by Discord, when signature mismatch
                    response = _flask_response_from_interaction(
                        interaction.InteractionResponse(
                            type=discord_types.InteractionResponseType.CHANNEL_MESSAGE_WITH_SOURCE,
                            data=interaction.InteractionResponseMessage(
                                content="Invalid message: Invalid signature"
                            )
                        ),
                        status=401
                    )
                    return response

                # Handle request
                try:
                    request_data = interaction.InteractionRequest(_app=self, **flask.request.json)  # type: ignore[arg-type]
                except TypeError as e:
                    # TypeError is raised when missing attributes
                    self._logger.warning(str(e))  # type: ignore[union-attr]
                    return _flask_response_from_interaction(
                        interaction.InteractionResponse(
                            type=discord_types.InteractionResponseType.CHANNEL_MESSAGE_WITH_SOURCE,
                            data=interaction.InteractionResponseMessage(
                                content="Invalid message: missing attributes"
                            )
                        ),
                        status=400
                    )

                if request_data.type == discord_types.InteractionType.PING:
                    self._logger.info("Ping event detected.")  # type: ignore[union-attr]
                    return _flask_response_from_interaction(
                        interaction.InteractionResponse(type=discord_types.InteractionCallbackType.PONG)
                    )

                elif request_data.type == discord_types.InteractionType.APPLICATION_COMMAND:
                    if request_data.data.name in self._command_list:  # type: ignore[union-attr, operator]
                        self._logger.info(  # type: ignore[union-attr]
                            "'%s' function is called using command '%s'.",
                            self._command_list[request_data.data.name]["function"].__name__,  # type: ignore[union-attr, index]
                            request_data.data.name  # type: ignore[union-attr]
                        )
                        command_response: interaction.InteractionResponse = \
                            self._command_list[request_data.data.name]["function"](request_data)  # type: ignore[union-attr, index]
                        return _flask_response_from_interaction(command_response)
                    self._logger.warning(  # type: ignore[union-attr]
                        "Handler for command '%s' is not found.",
                        request_data.data.name  # type: ignore[union-attr]
                    )
                    return _flask_response_from_interaction(
                        interaction.InteractionResponse(
                            type=discord_types.InteractionCallbackType.CHANNEL_MESSAGE_WITH_SOURCE,
                            data=interaction.InteractionResponseMessage(
                                content="No such command"
                            )
                        ),
                        status=404
                    )

                elif request_data.type == discord_types.InteractionType.MESSAGE_COMPONENT:
                    # TODO: implement here
                    return _flask_response_from_interaction(
                        interaction.InteractionResponse(
                            type=discord_types.InteractionResponseType.CHANNEL_MESSAGE_WITH_SOURCE,
                            data=interaction.InteractionResponseMessage(
                                content="Function not implemented"
                            )
                        ),
                        status=501
                    )
                elif request_data.type == discord_types.InteractionType.APPLICATION_COMMAND_AUTOCOMPLETE:
                    # TODO: implement here
                    return _flask_response_from_interaction(
                        interaction.InteractionResponse(
                            type=discord_types.InteractionResponseType.CHANNEL_MESSAGE_WITH_SOURCE,
                            data=interaction.InteractionResponseMessage(
                                content="Function not implemented"
                            )
                        ),
                        status=501
                    )
                elif request_data.type == discord_types.InteractionType.MODAL_SUBMIT:
                    # TODO: implement here
                    return _flask_response_from_interaction(
                        interaction.InteractionResponse(
                            type=discord_types.InteractionResponseType.CHANNEL_MESSAGE_WITH_SOURCE,
                            data=interaction.InteractionResponseMessage(
                                content="Function not implemented"
                            )
                        ),
                        status=501
                    )
                else:
                    # Unrecognized message type
                    self._logger.warning(  # type: ignore[union-attr]
                        "Invalid request: Unknown type: %s",
                        str(request_data.type)
                    )
                    return _flask_response_from_interaction(
                        interaction.InteractionResponse(
                            type=discord_types.InteractionResponseType.CHANNEL_MESSAGE_WITH_SOURCE,
                            data=interaction.InteractionResponseMessage(
                                content="Invalid interaction type"
                            )
                        ),
                        status=400
                    )

            self._handle_command = _handle_command

            # Register command router
            self._flask.route(
                self._endpoint if self._endpoint else "/",
                methods=["POST"]
            )(_handle_command)
            (self._previously_registered_commands, _) = self.call_api(  # type: ignore[misc]
                "GET",
                f"/applications/{self.id}/commands"
            )

    @property
    def _is_authorized(self) -> bool:
        return bool(self._bot_token and self._public_key)

    def call_api(
        self,
        method: str,
        path: str,
        headers: Union[requests.structures.CaseInsensitiveDict[Any], dict[str, str], None] = None,
        data: Optional[bytes] = None,
        json: Optional[object] = None,
        use_bot_token: bool = True
    ) -> Optional[Tuple[Any, requests.Response]]:
        if headers is None:
            headers = requests.structures.CaseInsensitiveDict()
        if isinstance(headers, dict):
            headers = requests.structures.CaseInsensitiveDict(headers)
        if "Authorization" not in headers:
            if use_bot_token:
                headers["Authorization"] = f"Bot {self._bot_token}"
        kwargs: Dict[str, Any] = {
            "method": method,
            "url": f"https://discord.com/api/v8{path}",
            "headers": headers
        }
        if data and json:
            raise RuntimeError("body and json are specified at the same time")
        if data:
            kwargs["data"] = data
        if json:
            kwargs["json"] = json
            if "Content-Type" not in kwargs["headers"]:
                kwargs["headers"]["Content-Type"] = "application/json"
        response = requests.request(**kwargs)
        response.raise_for_status()
        if response.status_code == 204:
            # 204 No Content
            return None, response
        if "Content-Type" in response.headers and response.headers["Content-Type"].lower() == "application/json":
            return response.json(), response  # type: ignore[no-any-return]
        else:
            raise RuntimeError(f"Server responded with unexpected content type: {response.headers['Content-Type']}")

    def application_command(self, options: interaction.ApplicationCommand, register_on_change: bool = True) -> Callable[[Any], Any]:
        """
            Define the following function as command handler.
            The command spec, , is defined as "option" parameter.

            This decorator can also register the new spec if change has been detected.
            Set "register_on_change" to False to disable it.

            Usage example:

            ```
            @app.application_command(command_spec, register_on_change=False)
            def my_command(discord_request):
                # Do things
                ...
            ```
        """
        if not self._is_authorized:
            raise RuntimeError("Unable to register a command that you have no control of. (Missing public_key and bot_token)")

        def decorator(
            function: Callable[[interaction.InteractionRequest], interaction.InteractionResponse]
        ) -> Callable[[interaction.InteractionRequest], interaction.InteractionResponse]:
            self._command_list[options.name] = {  # type: ignore[index]
                "function": function,
                "options": options
            }

            same_command_spec = False
            prev_option = next(
                (command for command in self._previously_registered_commands  # type: ignore[union-attr]
                    if command['name'] == options.name),
                None
            )
            if prev_option is not None:
                # Is a registered command, detect if command spec has changed.
                prev_option = interaction.ApplicationCommand(**prev_option)
                new_options = interaction.ApplicationCommand(**(
                    asdict(prev_option, dict_factory=asdict_ignore_none) |
                    asdict(options, dict_factory=asdict_ignore_none)))
                same_command_spec = (new_options == prev_option)
            if not same_command_spec:
                if prev_option is None:
                    self._logger.info("Registering new command %s", options.name)  # type: ignore[union-attr]
                    (_, response) = self.call_api(  # type: ignore[misc]
                        method="POST",
                        path=f"/applications/{self.id}/commands",
                        headers={"Content-Type": "application/json"},
                        json=asdict(options, dict_factory=asdict_ignore_none)
                    )
                    self._logger.info(f"Registration retruned with status code {response.status_code}")  # type: ignore[union-attr]
                else:
                    self._logger.info("Command specification has changed: %s", options.name)  # type: ignore[union-attr]
                    self._logger.debug("Before %s", repr(prev_option))  # type: ignore[union-attr]
                    self._logger.debug("After %s", repr(options))  # type: ignore[union-attr]
                    if register_on_change:
                        self._logger.info("Registering command %s", options.name)  # type: ignore[union-attr]
                        (_, response) = self.call_api(  # type: ignore[misc]
                            method="PATCH",
                            path=f"/applications/{self.id}/commands/{prev_option.id}",
                            headers={"Content-Type": "application/json"},
                            json=asdict(options, dict_factory=asdict_ignore_none)
                        )
                        self._logger.info(f"Registration retruned with status code {response.status_code}")  # type: ignore[union-attr]
            return function
        return decorator

    def run(self, *args: Any, **kwargs: Any) -> None:
        """
            Starts web server for incoming requests.
        """
        return self._flask.run(*args, **kwargs)

    @classmethod
    def from_basic_data(
        cls,
        id: discord_types.Snowflake,
        public_key: str,
        bot_token: str,
        endpoint: str = "/"
    ) -> Self:  # type: ignore[valid-type]
        appinfo_response = requests.request(
            "GET",
            "https://discord.com/api/v8/oauth2/applications/@me",
            headers={
                "Authorization": f"Bot {bot_token}"
            }
        )
        appinfo_response.raise_for_status()
        app_obj = appinfo_response.json()
        if app_obj['id'] == id:
            return cls(
                _bot_token=bot_token,
                _public_key=public_key,
                _endpoint=endpoint,
                **app_obj
            )
        raise ValueError(f"Discord responded with different application ID \"{app_obj['id']}\". Expected: \"{id}\"")


PartialApplication = Application




from dataclasses import dataclass
from typing import Optional, List

from . import discord_types
from . import user as user_module


@dataclass
class Sticker(discord_types.DiscordDataClass):
    id: discord_types.Snowflake
    name: str
    description: str
    tags: str
    type: discord_types.StickerType
    format_type: discord_types.StickerFormatType
    pack_id: Optional[discord_types.Snowflake] = None
    asset: Optional[str] = None  # Deprecated
    available: Optional[bool] = None
    guild_id: Optional[discord_types.Snowflake] = None
    user: Optional['user_module.User'] = None
    sort_value: Optional[int] = None

    def __post_init__(self) -> None:
        self.type = discord_types.StickerType(self.type)
        self.format_type = discord_types.StickerFormatType(self.format_type)
        if isinstance(self.user, dict):
            self.user = user_module.User(**self.user)  # type: ignore[unreachable]


@dataclass
class StickerItem(discord_types.DiscordDataClass):
    id: discord_types.Snowflake
    name: str
    format_type: discord_types.StickerFormatType

    def __post_init__(self) -> None:
        self.format_type = discord_types.StickerFormatType(self.format_type)


@dataclass
class StickerPack(discord_types.DiscordDataClass):
    id: discord_types.Snowflake
    stickers: List[Sticker]
    name: str
    sku_id: discord_types.Snowflake
    description: str
    cover_sticker_id: Optional[discord_types.Snowflake] = None
    banner_asset_id: Optional[discord_types.Snowflake] = None

    def __post_init__(self) -> None:
        self.stickers = [
            Sticker(**sticker) if isinstance(sticker, dict) else sticker
            for sticker in self.stickers
        ]




from dataclasses import dataclass
from typing import List, Optional
from . import discord_types
from . import user as user_module


@dataclass
class Emoji(discord_types.DiscordDataClass):
    id: discord_types.Snowflake
    name: Optional[str] = None
    roles: Optional[List[discord_types.Snowflake]] = None
    user: Optional['user_module.User'] = None
    require_colons: Optional[bool] = None
    managed: Optional[bool] = None
    animated: Optional[bool] = None
    available: Optional[bool] = None

    def __post_init__(self) -> None:
        if isinstance(self.user, dict):
            self.user = user_module.User(**self.user)  # type: ignore[unreachable]




from dataclasses import dataclass

from . import discord_types


@dataclass
class StageInstance(discord_types.DiscordDataClass):
    id: discord_types.Snowflake
    guild_id: discord_types.Snowflake
    channel_id: discord_types.Snowflake
    topic: str
    privacy_level: int
    discoverable_disabled: bool
    guild_scheduled_event_id: discord_types.Snowflake




from dataclasses import dataclass
from typing import List, Optional, Any

from . import discord_types
from . import user as user_module
from . import emoji
from . import channel
from . import voice
from . import sticker
from . import stage_instance
from . import guild_scheduled_event
from . import presence


@dataclass
class RoleTags(discord_types.DiscordDataClass):
    bot_id: Optional[discord_types.Snowflake] = None
    integration_id: Optional[discord_types.Snowflake] = None
    # Type of premium_sucscriver is "null" in the documentation
    premium_subscriber: Optional[Any] = None


@dataclass
class Role(discord_types.DiscordDataClass):
    """
        Discord role
    """
    id: discord_types.Snowflake
    name: str
    color: int
    hoist: bool
    position: int
    permissions: str
    managed: bool
    mentionable: bool
    icon: Optional[str] = None
    unicode_emoji: Optional[str] = None
    tags: Optional[RoleTags] = None

    def __post_init__(self) -> None:
        if isinstance(self.tags, dict):
            self.tags = RoleTags(**self.tags)  # type: ignore[unreachable]


@dataclass
class WelcomeScreenChannel(discord_types.DiscordDataClass):
    channel_id: discord_types.Snowflake
    description: str
    emoji_id: discord_types.Snowflake
    emoji_name: discord_types.Snowflake


@dataclass
class WelcomeScreen(discord_types.DiscordDataClass):
    description: str
    welcome_channels: List[WelcomeScreenChannel]

    def __post_init__(self) -> None:
        self.welcome_channels = [
            WelcomeScreenChannel(**wsc) if isinstance(wsc, dict) else wsc
            for wsc in self.welcome_channels
        ]


@dataclass
class PartialGuild(discord_types.DiscordDataClass):
    name: str
    roles: List[Role]
    channels: List[channel.Channel]
    verification_level: discord_types.VerificationLevel
    default_message_notifications: discord_types.DefaultMessageNotificationLevel
    explicit_content_filter: discord_types.ExplicitContentFilterLevel

    def __post_init__(self) -> None:
        self.roles = [
            Role(**r) if isinstance(r, dict) else r
            for r in self.roles
        ]
        self.channels = [
            channel.Channel(**c) if isinstance(c, dict) else c
            for c in self.channels
        ]
        self.verification_level = discord_types.VerificationLevel(self.verification_level)
        self.default_message_notifications = discord_types.DefaultMessageNotificationLevel(self.default_message_notifications)
        self.explicit_content_filter = discord_types.ExplicitContentFilterLevel(self.explicit_content_filter)


@dataclass
class Guild(PartialGuild):
    id: discord_types.Snowflake
    icon: str
    splash: str
    discovery_splash: str
    owner_id: discord_types.Snowflake
    afk_channel_id: discord_types.Snowflake
    afk_timeout: int
    emojis: List[emoji.Emoji]
    features: List[discord_types.GuildFeature]
    mfa_level: discord_types.MFALevel
    application_id: discord_types.Snowflake
    system_channel_id: discord_types.Snowflake
    system_channel_flags: discord_types.SystemChannelFlags
    rules_channel_id: discord_types.Snowflake
    vanity_url_code: str
    description: str
    banner: str
    premium_tier: discord_types.GuildPremiumTier
    preferred_locale: str
    public_updates_channel_id: discord_types.Snowflake
    nsfw_level: discord_types.GuildNSFWLevel
    premium_progress_bar_enabled: bool
    icon_hash: Optional[str] = None
    owner: Optional[bool] = None
    permissions: Optional[str] = None
    region: Optional[str] = None
    widget_enabled: Optional[bool] = None
    widget_channel_id: Optional[discord_types.Snowflake] = None
    joined_at: Optional[str] = None
    large: Optional[bool] = None
    unavailable: Optional[bool] = None
    member_count: Optional[int] = None
    voice_states: Optional[List[voice.PartialVoiceState]] = None
    members: Optional[List['GuildMember']] = None
    threads: Optional[List[channel.Channel]] = None
    presences: Optional[List[presence.PresenceUpdateEvent]] = None
    max_presences: Optional[int] = None
    max_members: Optional[int] = None
    premium_subscription_count: Optional[int] = None
    max_video_channel_users: Optional[int] = None
    approximate_member_count: Optional[int] = None
    approximate_presence_count: Optional[int] = None
    welcome_screen: Optional[WelcomeScreen] = None
    stage_instances: Optional[List[stage_instance.StageInstance]] = None
    stickers: Optional[List[sticker.Sticker]] = None
    guild_scheduled_events: Optional[List[guild_scheduled_event.GuildScheduledEvent]] = None

    def __post_init__(self) -> None:
        super().__post_init__()
        self.emojis = [
            emoji.Emoji(**e) if isinstance(e, dict) else e
            for e in self.emojis
        ]
        self.features = [
            discord_types.GuildFeature(f) if isinstance(f, str) else f
            for f in self.features
        ]
        self.mfa_level = discord_types.MFALevel(self.mfa_level)
        self.system_channel_flags = discord_types.SystemChannelFlags(self.system_channel_flags)
        self.premium_tier = discord_types.GuildPremiumTier(self.premium_tier)
        self.nsfw_level = discord_types.GuildNSFWLevel(self.nsfw_level)
        if isinstance(self.voice_states, list):
            self.voice_states = [
                voice.PartialVoiceState(**vs) if isinstance(vs, dict) else vs
                for vs in self.voice_states
            ]
        if isinstance(self.members, list):
            self.members = [
                GuildMember(**gm) if isinstance(gm, dict) else gm
                for gm in self.members
            ]
        if isinstance(self.threads, list):
            self.threads = [
                channel.Channel(**t) if isinstance(t, dict) else t
                for t in self.threads
            ]
        if isinstance(self.welcome_screen, dict):
            self.welcome_screen = WelcomeScreen(**self.welcome_screen)  # type: ignore[unreachable]
        if isinstance(self.stage_instances, list):
            self.stage_instances = [
                stage_instance.StageInstance(**si) if isinstance(si, dict) else si
                for si in self.stage_instances
            ]
        if isinstance(self.stickers, list):
            self.stickers = [
                sticker.Sticker(**s) if isinstance(s, dict) else s
                for s in self.stickers
            ]
        if isinstance(self.guild_scheduled_events, list):
            self.guild_scheduled_events = [
                guild_scheduled_event.GuildScheduledEvent(**gse) if isinstance(gse, dict) else gse
                for gse in self.guild_scheduled_events
            ]
        if isinstance(self.presences, list):
            self.presences = [
                presence.PresenceUpdateEvent(**p) if isinstance(p, dict) else p
                for p in self.presences
            ]


@dataclass
class GuildPreview(discord_types.DiscordDataClass):
    id: discord_types.Snowflake
    name: str
    icon: str
    splash: str
    discovery_splash: str
    emojis: List[emoji.Emoji]
    features: List[discord_types.GuildFeature]
    approximate_member_count: int
    approximate_presence_count: int
    description: str
    stickers: List[sticker.Sticker]

    def __post_init__(self) -> None:
        self.emojis = [
            emoji.Emoji(**e) if isinstance(e, dict) else e
            for e in self.emojis
        ]
        self.features = [
            discord_types.GuildFeature(feature) if isinstance(feature, str) else feature
            for feature in self.features
        ]
        self.stickers = [
            sticker.Sticker(**s) if isinstance(s, dict) else s
            for s in self.stickers
        ]


@dataclass
class GuildWidgetSettings(discord_types.DiscordDataClass):
    enabled: bool
    channel_id: discord_types.Snowflake


@dataclass
class GetGuildWidget(discord_types.DiscordDataClass):  # The name is not an typo, unless the API documentation is.
    id: discord_types.Snowflake
    name: str
    instant_invite: str
    channels: List[channel.PartialChannel]
    members: List['user_module.PartialUser']  # The documentation show here uses partial user for some reason
    presence_count: int

    def __post_init__(self) -> None:
        self.channels = [
            channel.PartialChannel(**c) if isinstance(c, dict) else c
            for c in self.channels
        ]
        self.members = [
            user_module.PartialUser(**m) if isinstance(m, dict) else m
            for m in self.members
        ]


@dataclass
class PartialGuildMember(discord_types.DiscordDataClass):
    """
        Guild member but without user, deaf and mute properties.
    """
    joined_at: str
    roles: List[discord_types.Snowflake]
    nick: Optional[str] = None
    avatar: Optional[str] = None
    premium_since: Optional[bool] = None
    pending: Optional[bool] = None
    permissions: Optional[str] = None
    communication_disabled_until: Optional[str] = None

    # The following attributes are undocumented in Discord API documentation
    flags: Optional[Any] = None
    is_pending: Optional[Any] = None


@dataclass
class GuildMember(PartialGuildMember):
    """
        Guild member
    """
    user: Optional['user_module.User'] = None
    deaf: bool = False
    mute: bool = False

    def __post_init__(self) -> None:
        # Parent class didn't define __post_init__ function, no need to execute.
        if isinstance(self.user, dict):
            self.user = user_module.User(**self.user)  # type: ignore[unreachable]


@dataclass
class IntegrationAccount(discord_types.DiscordDataClass):
    id: str
    name: str


@dataclass
class IntegrationApplication(discord_types.DiscordDataClass):
    id: discord_types.Snowflake
    name: str
    icon: str
    description: str
    bot: Optional['user_module.User']

    def __post_init__(self) -> None:
        if isinstance(self.bot, dict):
            self.bot = user_module.User(**self.bot)  # type: ignore[unreachable]


@dataclass
class Integration(discord_types.DiscordDataClass):
    id: discord_types.Snowflake
    name: str
    type: str
    account: IntegrationAccount
    enabled: Optional[bool] = None
    syncing: Optional[bool] = None
    role_id: Optional[discord_types.Snowflake] = None
    enable_emoticons: Optional[bool] = None
    expire_behavior: Optional[discord_types.IntegrationExpireBehavior] = None
    expire_grace_period: Optional[int] = None
    user: Optional['user_module.User'] = None
    synced_at: Optional[str] = None
    subscriber_count: Optional[int] = None
    revoked: Optional[bool] = None
    application: Optional[IntegrationApplication] = None

    def __post_init__(self) -> None:
        if isinstance(self.account, dict):
            self.account = IntegrationAccount(**self.account)  # type: ignore[unreachable]
        if isinstance(self.expire_behavior, int):
            self.expire_behavior = discord_types.IntegrationExpireBehavior(self.expire_behavior)
        if isinstance(self.user, dict):
            self.user = user_module.User(**self.user)  # type: ignore[unreachable]
        if isinstance(self.application, dict):
            self.application = IntegrationApplication(**self.application)  # type: ignore[unreachable]


@dataclass
class Ban(discord_types.DiscordDataClass):
    reason: str
    user: 'user_module.User'

    def __post_init__(self) -> None:
        if isinstance(self.user, dict):
            self.user = user_module.User(**self.user)  # type: ignore[unreachable]




from dataclasses import dataclass
from typing import Optional

from . import discord_types
from . import user as user_module
from . import guild as guild_module


@dataclass
class Webhook(discord_types.DiscordDataClass):
    id: discord_types.Snowflake
    type: discord_types.WebhookType
    guild_id: discord_types.Snowflake
    channel_id: discord_types.Snowflake
    name: str
    avatar: str
    application_id: discord_types.Snowflake
    user: Optional[user_module.User] = None
    token: Optional[str] = None
    source_guild: Optional[guild_module.PartialGuild] = None
    url: Optional[str] = None

    def __post_init__(self) -> None:
        self.type = discord_types.WebhookType(self.type)
        if isinstance(self.user, dict):
            self.user = user_module.User(**self.user)  # type: ignore[unreachable]
        if isinstance(self.source_guild, dict):
            self.source_guild = guild_module.PartialGuild(**self.source_guild)  # type: ignore[unreachable]



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['discord_app']

package_data = \
{'': ['*']}

install_requires = \
['Flask>=2.1.1,<3.0.0', 'PyNaCl>=1.4.0,<2.0.0', 'requests>=2.27.1,<3.0.0']

setup_kwargs = {
    'name': 'discord-app',
    'version': '0.0.1a1',
    'description': 'Discord Interaction Application framework for Python',
    'long_description': '# Discord Interaction Application framework for Python\n\n![Tests](https://github.com/jacky9813/discord_app/actions/workflows/tests.yaml/badge.svg) [![Pypi](https://badge.fury.io/py/discord-app.svg)](https://pypi.org/project/discord-app/)\n\n> In early development.\n> \n> The specification and documentation may change without notification.\n> \n> Any new ideas are welcomed.\n\nA [Flask](https://flask.palletsprojects.com/) based Discord Application framework.\n\n## System Requirements\n\n* Python 3.9+\n* (Recommended) [virtualenv](https://virtualenv.pypa.io/en/latest/)\n\n## Install from PyPI\n\nInstall and update using [pip](https://pip.pypa.io/en/stable/getting-started/):\n\n```bash\n $ pip install discord-app\n```\n\n## Known Limitation\n\n* After command specification updated (including new command), Discord clients may require up to an hour to update its command list.\n\n## Sample Server Application\n```python\n#!/usr/bin/env python3\n# app.py\nimport discord_app\n\nfrom . import app_config\n\nPORT = "8080"\n\napp = discord_app.Application.from_basic_data(\n    id=app_config.APPLICATION_ID,\n    public_key=app_config.PUBLIC_KEY,\n    bot_token=app_config.BOT_TOKEN,\n    # Set Discord interaction endpoint URL to http(s)://<YOUR-ADDRESS>:<PORT>/<ENDPOINT>\n    endpoint="/api/discord/command"\n)\n\napp._flask.testing = True\napp._logger.setLevel(logging.DEBUG)\n\n\n@app.application_command(\n    options=discord_app.ApplicationCommand(\n        name="version",\n        name_localizations={\n            "zh-TW": "顯示程式版本"\n        },\n        type=discord_app.ApplicationCommandType.CHAT_INPUT,\n        description="Show application version",\n        description_localizations={\n            "zh-TW": "顯示程式版本"\n        }\n    ),\n    register_on_change=True  # Register command on specification change or is new command.\n)\ndef show_version(_: discord_app.InteractionRequest) -> discord_app.InteractionResponse:\n    return discord_app.InteractionResponse(\n        type=discord_app.InteractionResponseType.CHANNEL_MESSAGE_WITH_SOURCE,\n        data=discord_app.InteractionResponseMessage(\n            content="test application/0.0.1"\n        )\n    )\n\napp.run(\n    host="0.0.0.0",\n    port=PORT\n)\n```\n\n```bash\n $ python3 app.py\n```\n\n## Links\n* [Official API Documentation](https://discord.com/developers/docs/)\n',
    'author': 'JackyCCC',
    'author_email': 'jacky9813@hotmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jacky9813/discord_app',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4',
}


setup(**setup_kwargs)




from dataclasses import dataclass
from typing import Optional, Any, List

from . import discord_types
from . import guild as guild_module


@dataclass
class User(discord_types.DiscordDataClass):
    """
        Discord user
    """
    id: discord_types.Snowflake
    username: str
    discriminator: str
    avatar: str
    bot: Optional[bool] = None
    system: Optional[bool] = None
    mfa_enabled: Optional[bool] = None
    banner: Optional[str] = None
    accent_color: Optional[int] = None
    locale: Optional[str] = None
    verified: Optional[bool] = None
    email: Optional[str] = None
    flags: Optional[discord_types.UserFlag] = None
    premium_type: Optional[discord_types.PremiumTypes] = None
    public_flags: Optional[discord_types.UserFlag] = None

    # The following attributes are undocumented
    avatar_decoration: Optional[Any] = None

    def __post_init__(self) -> None:
        if isinstance(self.flags, int):
            self.flags = discord_types.UserFlag(self.flags)
        if isinstance(self.premium_type, int):
            self.premium_type = discord_types.PremiumTypes(self.premium_type)
        if isinstance(self.public_flags, int):
            self.public_flags = discord_types.UserFlag(self.public_flags)


PartialUser = User


@dataclass
class Connection(discord_types.DiscordDataClass):
    id: str
    name: str
    type: str
    verified: bool
    friend_sync: bool
    show_activity: bool
    visibility: discord_types.Visibility
    revoked: Optional[bool] = None
    integrations: Optional[List[guild_module.Integration]] = None

    def __post_init__(self) -> None:
        self.visibility = discord_types.Visibility(self.visibility)
        if isinstance(self.integrations, list):
            self.integrations = [
                guild_module.Integration(**i) if isinstance(i, dict) else i
                for i in self.integrations
            ]




from dataclasses import dataclass
from typing import Optional, List

from . import discord_types
from . import guild as guild_module
from . import channel as channel_module
from . import user as user_module
from . import application as application_module
from . import guild_scheduled_event as gse_module


@dataclass
class InviteStageInstance(discord_types.DiscordDataClass):
    members: List[guild_module.PartialGuildMember]
    participant_count: int
    speaker_count: int
    topic: str

    def __post_init__(self) -> None:
        self.members = [
            guild_module.PartialGuildMember(**m) if isinstance(m, dict) else m
            for m in self.members
        ]


@dataclass
class Invite(discord_types.DiscordDataClass):
    code: str
    channel: channel_module.PartialChannel
    guild: Optional[guild_module.PartialGuild] = None
    inviter: Optional[user_module.User] = None
    target_type: Optional[discord_types.InviteTargetType] = None
    target_user: Optional[user_module.User] = None
    target_application: Optional[application_module.PartialApplication] = None
    approximate_presence_count: Optional[int] = None
    approximate_member_count: Optional[int] = None
    expires_at: Optional[str] = None
    stage_instance: Optional[InviteStageInstance] = None
    guild_scheduled_event: Optional[gse_module.GuildScheduledEvent] = None

    def __post_init__(self) -> None:
        if isinstance(self.channel, dict):
            self.channel = channel_module.PartialChannel(**self.channel)  # type: ignore[unreachable]
        if isinstance(self.guild, dict):
            self.guild = guild_module.PartialGuild(**self.guild)  # type: ignore[unreachable]
        if isinstance(self.inviter, dict):
            self.inviter = user_module.User(**self.inviter)  # type: ignore[unreachable]
        if isinstance(self.target_type, int):
            self.target_type = discord_types.InviteTargetType(self.target_type)
        if isinstance(self.target_user, dict):
            self.target_user = user_module.User(**self.target_user)  # type: ignore[unreachable]
        if isinstance(self.target_application, dict):
            self.target_application = application_module.PartialApplication(**self.target_application)  # type: ignore[unreachable]
        if isinstance(self.stage_instance, dict):
            self.stage_instance = InviteStageInstance(**self.stage_instance)  # type: ignore[unreachable]
        if isinstance(self.guild_scheduled_event, dict):
            self.guild_scheduled_event = gse_module.GuildScheduledEvent(**self.guild_scheduled_event)  # type: ignore[unreachable]


@dataclass
class InviteMetadata(discord_types.DiscordDataClass):
    uses: int
    max_uses: int
    max_age: int
    temporary: bool
    created_at: str




from dataclasses import dataclass

from . import discord_types
from . import user as user_module
from . import guild as guild_module


@dataclass
class GuildTemplate(discord_types.DiscordDataClass):
    code: str
    name: str
    description: str
    usage_count: int
    creator_id: discord_types.Snowflake
    creator: user_module.User
    created_at: str
    updated_at: str
    source_guild_id: discord_types.Snowflake
    serialized_source_guild: guild_module.PartialGuild
    is_dirty: bool

    def __post_init__(self) -> None:
        if isinstance(self.creator, dict):
            self.creator = user_module.User(**self.creator)  # type: ignore[unreachable]
        if isinstance(self.serialized_source_guild, dict):
            self.serialized_source_guild = guild_module.PartialGuild(**self.serialized_source_guild)  # type: ignore[unreachable]
